# -*- coding: utf-8 -*-

# CTRIA3     14524       5    2453    2454    245136.264230.000000        +132061
# +132061                 0.0001000.0001000.000100
               

filename="lynx.dat"

import enum

class State(enum.IntEnum):
    DEFAULT = 0
    NODE = 1

ntags = []
ncoords = []

groups = {}


state = State.DEFAULT

ntri=0
with open(filename) as infile:
    for nline, l in enumerate(infile.readlines()):
        line = l.rstrip()
        if state==State.NODE:
            node_z=float(line[8:24])
            ncoords[-1][2] = node_z
            state = State.DEFAULT
        if line[:4]=='GRID':
            node_no=int(line[8:24])
            node_x=float(line[41:56])
            node_y=float(line[56:72])
            # print(node_no, node_x, node_y)
            ntags.append(node_no)
            ncoords.append( [ node_x, node_y, 0.0 ])
            state = State.NODE
        if line[:6]=='CTRIA3':
            gr=int(line[17:25])
            n1=int(line[25:33])
            n2=int(line[33:41])
            n3=int(line[41:48])
            groups.setdefault(gr, []).append([n1, n2, n3])
            ntri+=1

print(f'mesh with {len(ntags)} and {ntri} triangles')

for gno,elms in groups.items():
    print(f"group #{gno} contains {len(elms)} triangles")

# create nodemap
nodemap = [-1] * (max(ntags)+1)
for i, tag in enumerate(ntags):
    nodemap[tag] = i

# create polydata
import vtk
colors = vtk.vtkNamedColors()
cols = ['white', 'green', 'red', 'blue', 'yellow', 'purple', 'cyan']
class Mesh:
    def __init__(self):
        self.polydata = None
        self.mapper = None
        self.actor = None

meshes = {}

for gno,group in groups.items():
    polydata = vtk.vtkPolyData()
    points   = vtk.vtkPoints()
    polys    = vtk.vtkCellArray()

    for i,x in enumerate(ncoords):
        points.InsertPoint(i, x[0], x[1], x[2])

    for elms in group:
        polys.InsertNextCell(3)
        try:
            polys.InsertCellPoint(nodemap[elms[0]])
            polys.InsertCellPoint(nodemap[elms[1]])
            polys.InsertCellPoint(nodemap[elms[2]])
        except IndexError:
            print(f'{elms}')

    polydata.SetPoints(points)
    polydata.SetPolys(polys)


    # display mesh

    mapper = vtk.vtkDataSetMapper()
    mapper.SetInputData(polydata)
    actor = vtk.vtkActor()
    actor.SetMapper(mapper)
    actor.GetProperty().SetColor(colors.GetColor3d(cols[ gno % len(cols) ]))
    # actor.GetProperty().EdgeVisibilityOn()

    meshes[gno] = Mesh()
    meshes[gno].polydata = polydata
    meshes[gno].mapper = mapper
    meshes[gno].actor = actor
    

ren = vtk.vtkRenderer()
ren.SetBackground(colors.GetColor3d('cobalt') )

renWin = vtk.vtkRenderWindow()
renWin.SetSize(640, 480)
renWin.AddRenderer(ren)
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

for gno,mesh in meshes.items():    
    ren.AddActor(mesh.actor)

ren.ResetCamera()

iren.Initialize()
renWin.Render()
iren.Start()
